% InSegtFibre_3D
% Written by Monica Jane Emerson, May 2018.
% MIT license

%% Step 0: Add paths of Insegt GUI, other functions and scripts, and prepare workspace
close all,
clear all,
addpath(genpath('./texture_gui'))
addpath('./scripts')
addpath('./functions')

disp('Step 0 completed')

%% Step 1: Locate folder with the CT data
close all
indicate_dataFolder

disp('Step 1 completed')

%% Step 2: Load and visualise the data
close all
[V,depth] = loadVolumeRoI(path_volumeFolder);
figure, imshow3D(V) %visualise the loaded RoI

disp('Step 2 completed')

%% Step 3: Compute a dictionary model from the CT scan 
close all
setup_dict %set the parameters of the dictionary model
create_dict %learn the dictionary model from training data

disp('Step 3 completed')

%% Step 4: Obtain the centre-class probabilistic segmentation for a slice
close all
% USER INPUT: Load a saved dictionary
[filename,pathname] = uigetfile('*.mat','Choose dictionary','../data/saved_data/dictionaries/dictionary');
load([pathname,filename]);

% USER INPUT: Choose slice
prompt = ['Choose a slice to segment: [1,', num2str(size(V,3)),']:'];
x = inputdlg(prompt,'User input',[1,30],{num2str(round(size(V,3)/4))}); 
sliceAnalys = str2double(x{1});
h = msgbox('Segmenting image...');
[~,allP] = process_image(V(:,:,sliceAnalys),dictionary);
delete(h)
Pcentre = allP(:,:,2);

%Visualise for each pixel its probabily of belonging to a fibre centre region
h = figure(1), %h.Units = 'normalized'; h.Position = [0 0 1 1]; 
subplot(1,2,1), imagesc(Pcentre), axis image, colormap gray,  colorbar, 
title('Probability of belonging to the central region of a fibre') 
figure(1), subplot(1,3,3), histogram(Pcentre(:)), 
ylabel('Count of probabilities'), xlabel('Range of probabilities') 

disp('Step 4 completed')

%% Step 5: Obtain the fibre centres by thresholding the probabilistic segmentation
% USER INPUT: threshold value for the probability map 
thresh = inputdlg('Choose threshold: ','User input',[1,20],{num2str(0.50)});

%Threshold probability map
areas_th = Pcentre > str2double(thresh{1}); 

%Calculate centroids
concomp = logical(areas_th);
c = regionprops(concomp,'centroid');
centrePoints_2D = cat(1,c.Centroid);
    
%Visualisations to evaluate the chosen threshold
figure(2), subplot(1,2,1), imagesc(V(:,:,sliceAnalys)), axis image, colormap gray, 
hold on, h = imagesc(ind2rgb(areas_th, cool(2))), set(h, 'AlphaData',0.5);
figure(2), subplot(1,2,2), imagesc(V(:,:,sliceAnalys)), axis image, colormap gray
hold on, plot(centrePoints_2D(:,1),centrePoints_2D(:,2),'r*');

disp('Step 5 completed')

%% Step 6: Detect fibre centres in the batch of slices forming the volume
close all
clear centrePoints, clear allP, clear c
% USER INPUT: Load a saved dictionary
[filename,pathname] = uigetfile('*.mat','Choose dictionary','../data/saved_data/dictionaries/dictionary_');
load([pathname,filename]);

% USER INPUT: threshold value for the probability map 
thresh = inputdlg('Choose threshold: ','User input',[1,20],{num2str(0.5)});
 
%Initialise variables
step = 1; 
slices = 1:step:size(V,3);

%Start timer
tstart = tic; 
h = waitbar(0,'Processing volume...');

for l = 1:numel(slices)
    %Compute the probabilistic segmentation
    [~,allP] = process_image(V(:,:,slices(l)),dictionary);
    %Save probability map
    P(:,:,l) = allP(:,:,2);
    %Compute the connected components of the thresholded probabilities
    c = regionprops(bwlabel(allP(:,:,2) > str2num(thresh{1})),'centroid');
    %Store the coordinates of the centre points
    centrePoints{l} = cat(1,c.Centroid);
    %Update the data processing wait bar
    waitbar(l/numel(slices),h)
end

%disp processing time
t_load = toc(tstart);
f = msgbox(['time to process volume: ' , num2str(t_load), 's']);
close(h)
waitfor(f)

%Compute the number of detected fibres per slice
clear len
close all
for l = 1:length(centrePoints)
    len(l) = size(centrePoints{l},1);
end
aux = str2double(depth);
figure(1), plot([aux(1):aux(3):aux(2)],len,'.'), axis tight, title('Nr. fibres per slice'), 
xlabel('slice number (1 is the top of the sample)')
ylabel('number of detected fibres')

%Visualise the detections in 3D
if ~exist('vx_size')
    vx_size = str2double(inputdlg(['Pixel size [', char(181),'m]: '],'User input',[1,30],{num2str(1.1)}));
end

for l = 1:step:length(centrePoints)
    figure(2), hold on, plot3(centrePoints{l}(:,1)*vx_size,centrePoints{l}(:,2)*vx_size,l*ones(size(centrePoints{l}(:,1)))*vx_size,'r.','MarkerSize',1)
end
h = figure(2), axis tight, view(3), axis equal
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')
aux = cellstr(num2str(vx_size*linspace(str2double(depth{1}),str2double(depth{2}),length(h.Children.ZTickLabel)+1)','%.f'));
h.Children.ZTickLabel = aux(2:end);

% USER INPUT: Save 3D detections
uisave('centrePoints','../data/saved_data/centreCoordinates/centreCoords_?.mat')

disp('Step 6 completed')

%% Step 7: Compute the 3D fibre trajectories via tracking
% USER INPUT: Load the saved 3D detections
[filename,pathname] = uigetfile('*.mat','Choose the set of 3D coordinates to track','../data/saved_data/centreCoordinates');
load([pathname,filename]);

% USER INPUT: How many pixels is a fibre allowed to move from one slice to the
% next? (function of the step, the diameter and the pixel size).
x = inputdlg('How many pixels is a fibre allowed to move from one slice to the next? (a function of the step, the diameter and the pixel size): ','User input',[1,65],{num2str(5)}); 
dist = str2num(x{1});

% USER INPUT: Should the tracking start with the detections of the bottom 
% or the top slice?
if ~exist('depth')
    depth = input_depthRoI(num_slices);
end
top = 'Top of the volume (slice 1)';
bottom = ['Bottom of the volume (slice ', depth{2},')'];
direct = questdlg('Where shall we start the fibres?','Tracking direction',top, bottom,bottom);

if strcmp(direct,bottom)
    m = length(centrePoints);
else 
    m = 1;
end

%Initialise variables
thisCt = centrePoints{m}; %we start tracking from the bottom of the volume (end slice)
f_x = nan(size(thisCt,1),length(centrePoints));
f_y = nan(size(thisCt,1),length(centrePoints));
f_x(:,1) = centrePoints{m}(:,1);
f_y(:,1) = centrePoints{m}(:,2);

%Starting time
tstart = tic;

%Track the coordinates in the z direction
for l = 2:length(centrePoints)
    % coordinates of next slice
    if strcmp(direct,bottom)
        ctNext = centrePoints{end-l+1};
    else
        ctNext = centrePoints{l};
    end
    
    % build kd-tree
    tree_down = KDTreeSearcher(ctNext);
    % search the kd-tree
    [id_down, d_down] = knnsearch(tree_down,thisCt,'K',1);
    % build kd-tree
    tree_up = KDTreeSearcher(thisCt);
    % search the kd-tree
    [id_up, d_up] = knnsearch(tree_up,ctNext,'K',1);
    
    % keep matches that are bidirectional
    id_top = unique(id_up(id_down));
    id_bottom = id_down(unique(id_up(id_down)));
    
    % keep matches that are closer than a distance
    idMatch = find(d_down(id_top) < dist);
    id_top_th = id_top(idMatch);
    id_bottom_th = id_bottom(idMatch);
    
    if strcmp(direct,bottom)
        f_x(id_top_th,l) = centrePoints{end-l+1}(id_bottom_th,1);
        f_y(id_top_th,l) = centrePoints{end-l+1}(id_bottom_th,2);
    else
        f_x(id_top_th,l) = centrePoints{l}(id_bottom_th,1);
        f_y(id_top_th,l) = centrePoints{l}(id_bottom_th,2);
    end
    
    %points to be matched with the next slice
    thisCt(id_top_th,:) = ctNext(id_bottom_th,:);
end
t_load = toc(tstart);
disp(['time to track coordinates along the volume: ' , num2str(t_load), 's']);

%Save track coordinates in the structure 'fibres'
fibres.x = f_x;
fibres.y = f_y;

%Visualise fibre tracks
% USER INPUT: How many fibres would you like to visualise (in percentage)?
questions = {'What percentage of the tracked fibres would you like to visualise (%)?: '};
x = inputdlg(questions,'User input',[1,40],{num2str(10)}); %ask for patch size

perc = str2double(x{1})/100;
fib_ind = randperm(size(fibres.x,1),round(perc*size(fibres.x,1))); %indices of fibre to plot
z = [1:size(fibres.x,2)]; %z-values

if ~exist('vx_size')
    vx_size = str2double(inputdlg(['Pixel size [', char(181),'m]: '],'User input',[1,30],{num2str(1.1)}));
end
figure(3),clf
for l= 1:numel(fib_ind)
	h = figure(3);
    hold on, plot3(fibres.x(fib_ind(l),:)*vx_size,fibres.y(fib_ind(l),:)*vx_size,z*vx_size,'LineWidth',1), 
end
axis tight, axis equal, view(3), 
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')
aux = cellstr(num2str(vx_size*linspace(str2double(depth{1}),str2double(depth{2}),length(h.Children.ZTickLabel)+1)','%.f'));
h.Children.ZTickLabel = aux(2:end);

% USER INPUT: Save fibre tracks
if strcmp(direct,bottom)
    direction = [depth{2},'to',depth{1}];
else
    direction = [depth{1},'to',depth{2}];
end
uisave('fibres',['../data/saved_data/fibreTracks/fibreTracks_dataSetX_',direction,'step',depth{3},'.mat'])

    