function [rect,im_rect] = selectFoV(slice_path)
%SELECTFOV is a user-friendly function to select a region of interest from a 2D image.
%   The user can choose to select interactively the region of interest (RoI) 
%   interactively with imcrop, or specify the coordinates of the RoI.
% by Monica Jane Emerson, March 2019, MIT License

%USER INPUT: SLICE_PATH, The location of the slice to crop the RoI from.
% Other user input given through interaction with windows.

autoManual = questdlg('How would you like to select the region of interest (RoI)?','Select analysis region','Interactively','I know the coordinates','Interactively');
im = imread(slice_path);

if strcmp(autoManual,'Interactively')
    [~,rect] = imcrop(im);
    rect = round(rect);
    rect(3:4) = rect(1:2) + rect(3:4) - [1,1];
else
    xmin = ['Lower limit for the x value [1, ',num2str(size(im,2)),']:'];
    xmax = ['Upper limit for the x value [1, ',num2str(size(im,2)),']:'];
    ymin = ['Lower limit for the y value [1, ',num2str(size(im,1)),']:'];
    ymax = ['Upper limit for the y value [1, ',num2str(size(im,1)),']:'];
    prompt = {xmin,ymin,xmax,ymax};
    dimRoI = inputdlg(prompt,'RoI in the cross-sectional direction', [1,60], {'200','200','1799','1799'});
    rect = str2double(dimRoI);
end
    im_rect = double(im(rect(2):rect(4),rect(1):rect(3)))/2^16;

end

